#ifndef LCD_H_
#define LCD_H_
//----------------------------------------------------------------------------------------
//
//		Parametry pracy sterownika
//
//----------------------------------------------------------------------------------------
// rozdzielczo wywietlacza LCD (wiersze/kolumny)
#define LCD_Y 2		// ilo wierszy wywietlacza LCD
#define LCD_X 16	// ilo kolumn wywietlacza LCD
#define USE_RW 0

//----------------------------------------------------------------------------------------
//
//		Ustawienia sprztowe pocze sterownika z mikrokontrolerem
//
//----------------------------------------------------------------------------------------
// tu konfigurujemy port i piny do jakich podczymy linie D7..D4 LCD
#define LCD_D7PORT  D
#define LCD_D7 1
#define LCD_D6PORT  D
#define LCD_D6 2
#define LCD_D5PORT  D
#define LCD_D5 3
#define LCD_D4PORT  D
#define LCD_D4 4


// tu definiujemy piny procesora do ktrych podczamy sygnay RS,RW, E
#define LCD_RSPORT B
#define LCD_RS 7

#define LCD_RWPORT C
#define LCD_RW 4

#define LCD_EPORT B
#define LCD_E 6
//------------------------------------------------  koniec ustawie sprztowych ---------------

#define USE_LCD_LOCATE	1			// ustawia kursor na wybranej pozycji Y,X (Y=0-3, X=0-n)

#define USE_LCD_CHAR 	1			// wysya pojedynczy znak jako argument funkcji

#define USE_LCD_STR_P 	1			// wysya string umieszczony w pamici FLASH
#define USE_LCD_STR_E 	1			// wysya string umieszczony w pamici FLASH

#define USE_LCD_INT 	1			// wywietla liczb dziesietn na LCD
#define USE_LCD_HEX 	1			// wywietla liczb szesnastkow na LCD

#define USE_LCD_DEFCHAR		1		// wysya zdefiniowany znak z pamici RAM
#define USE_LCD_DEFCHAR_P 	1		// wysya zdefiniowany znak z pamici FLASH
#define USE_LCD_DEFCHAR_E 	1		// wysya zdefiniowany znak z pamici EEPROM

#define USE_LCD_CURSOR_ON 		0	// obsuga wczania/wyczania kursora
#define USE_LCD_CURSOR_BLINK 	0	// obsuga wczania/wyczania migania kursora
#define USE_LCD_CURSOR_HOME 	0	// ustawia kursor na pozycji pocztkowej

//------------------------------------------------  koniec ustawie kompilacji ---------------

#if ( (LCD_Y == 4) && (LCD_X == 16) )
#define LCD_LINE1 0x00		// adres 1 znaku 1 wiersza
#define LCD_LINE2 0x28		// adres 1 znaku 2 wiersza
#define LCD_LINE3 0x14  	// adres 1 znaku 3 wiersza
#define LCD_LINE4 0x54  	// adres 1 znaku 4 wiersza
#else
#define LCD_LINE1 0x00		// adres 1 znaku 1 wiersza
#define LCD_LINE2 0x40		// adres 1 znaku 2 wiersza
#define LCD_LINE3 0x10  	// adres 1 znaku 3 wiersza
#define LCD_LINE4 0x50  	// adres 1 znaku 4 wiersza
#endif


// Makra upraszczajce dostp do portw
// *** PORT
#define PORT(x) SPORT(x)
#define SPORT(x) (PORT##x)
// *** PIN
#define PIN(x) SPIN(x)
#define SPIN(x) (PIN##x)
// *** DDR
#define DDR(x) SDDR(x)
#define SDDR(x) (DDR##x)


// Komendy sterujce
#define LCDC_CLS					0x01
#define LCDC_HOME					0x02
#define LCDC_ENTRY					0x04
	#define LCDC_ENTRYR					0x02
	#define LCDC_ENTRYL					0
	#define LCDC_MOVE					0x01
#define LCDC_ONOFF					0x08
	#define LCDC_DISPLAYON				0x04
	#define LCDC_CURSORON				0x02
	#define LCDC_CURSOROFF				0
	#define LCDC_BLINKON				0x01
#define LCDC_SHIFT					0x10
	#define LCDC_SHIFTDISP				0x08
	#define LCDC_SHIFTR					0x04
	#define LCDC_SHIFTL					0
#define LCDC_FUNC					0x20
	#define LCDC_FUNC8B					0x10
	#define LCDC_FUNC4B					0
	#define LCDC_FUNC2L					0x08
	#define LCDC_FUNC1L					0
	#define LCDC_FUNC5x10				0x04
	#define LCDC_FUNC5x7				0
#define LCDC_SET_CGRAM				0x40
#define LCDC_SET_DDRAM				0x80





// deklaracje funkcji na potrzeby innych moduw
void lcd_init(void);								// WCZONA na stae do kompilacji
void lcd_cls(void);									// WCZONA na stae do kompilacji
void lcd_str(char * str);							// WCZONA na stae do kompilacji

void lcd_locate(uint8_t y, uint8_t x);				// domylnie WCZONA z kompilacji w pliku lcd.c

void lcd_char(char c);								// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_str_P(char * str);							// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_str_E(char * str);							// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_int(int val);								// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_hex(int val);								// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_defchar(uint8_t nr, uint8_t *def_znak);	// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_defchar_P(uint8_t nr, uint8_t *def_znak);	// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_defchar_E(uint8_t nr, uint8_t *def_znak);	// domylnie wyczona z kompilacji w pliku lcd.c

void lcd_home(void);								// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_cursor_on(void);							// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_cursor_off(void);							// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_blink_on(void);							// domylnie wyczona z kompilacji w pliku lcd.c
void lcd_blink_off(void);							// domylnie wyczona z kompilacji w pliku lcd.c

#endif /* LCD_H_ */
